#ifndef UCIPARSER_H
#define UCIPARSER_H

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <map>

/*!
 * manages the communication with a UCI connection
 */
class UciParser
{
    using CommandHandler = void(UciParser::*)(const std::vector<std::string>&);
    static const std::map<std::string, CommandHandler> commandHandlers;

    static constexpr const char* UCIOK = "uciok";
    static constexpr const char* READYOK = "readyok";

public:
    UciParser(void) = default;
    virtual ~UciParser(void) = default;

    virtual int parse(std::istream& in, std::ostream& out);

    virtual int executeLine(const std::string& line);
    virtual int executeCommand(const std::string& command,
                               const std::vector<std::string>& args);

    virtual void sendCommand(const std::string& command,
        const std::vector<std::string>& args = {});

    virtual void uci            (const std::vector<std::string>& args);
    virtual void debug          (const std::vector<std::string>& args);
    virtual void isready        (const std::vector<std::string>& args);
    virtual void setoption      (const std::vector<std::string>& args);
    virtual void ucinewgame     (const std::vector<std::string>& args);

    /*!
     * Is needed for commands that should not trigger any action, but are
     * still valid commands. An example would be the "register" command,
     * since this chess engine does not have such a feature.
     */
    virtual void doNothing      (const std::vector<std::string>& args);
};

#endif // UCIPARSER_H
